///////////////////////////////////////////////////////////////////////////////
/*! 
 *  \file GLAppGPUTest.h
 *  \brief FragSniffer GPU Test class.
 *  
 *  The Froggy FragSniffer.
 *	This application has been made for testing GPU fragment scheduling among 
 *  threads processors.
 *  \author C. Crassin, cyril.crassin@inrialpes.fr
 *	\date 05/03/2008
 *  \version 0.2
 */
///////////////////////////////////////////////////////////////////////////////

#ifndef GLAppGPUTest_h
#define GLAppGPUTest_h


#include <NemoGraphics/GLApp.h>
#include <NemoGraphics/ShaderProgramGLSL.h>
#include <NemoGraphics/ResourcesManager.h>

#include <NemoGraphics/Timer.h>
#include <NemoGraphics/TimerGL.h>

#include <AntTweakBar.h>

using namespace NemoGraphics;

#define NUM_TESTS 2
#define NUM_FREEZE_POINTS 4
#define NUM_USER_POINTS 4


class GLAppGPUTest : public GLApp {

	static GLAppGPUTest *curGPUTestApp;

	//Mouse control
	Vector2i oldMousePos;
	bool mouseButtons[3];

	//Vector4f userParam;
	//Vector4f userParamDif;

	//Vector4f userInfo;

	Vector4i freezePoints[NUM_FREEZE_POINTS];
	bool freezePointsEnable[NUM_FREEZE_POINTS];
	int freezePointsNum[NUM_FREEZE_POINTS]; //only for interface
	int curFreezePoint;

	Vector2i userPoints[NUM_USER_POINTS];
	int curUserPoint;
	bool controlUserPoints;

	bool useBadLoop;
	int numBadLoopCycles;

	bool fastMode;
	bool showInfo;

	bool fullScreen;
	//bool swapBuff;

	bool blockingEnable;

	int curTest;
	Vector4i winInfo;
	Vector2i screenSize;

	Vector2i loopsCount;
	Vector2i loopsCountDif;

	GLuint primitiveType;

	Vector2i miniTilesSize;
	int miniTilesSizeDif;

	Vector2i primitivesStep;
	Vector2i primitivesSize;
	int primitivesRepeat;
	int primitivesRepeatMode;

	int samePointMode;
	bool primitivesRestartMode;

	int lineOrientation;

	int frameCounter;

	int moveMode;
	bool useBigTriangle;

	GLuint curDrawBuffer;
	GLuint curClearBuffer;

	////////Test Cubes Support//////////
	ShaderProgramGLSL *gpuTestProg;
	ShaderGLSL *gpuTestVS;
	ShaderGLSL *gpuTestFS;
	/////////////////////


	void drawTest();

	void drawTest0();
	void drawTest1();

	////////////////
	bool interfMode;
	TwBar *optBarMain;
	TwBar *optBarPoints;

	static void TW_CALL callbackPUS(void *clientData) { 
		curGPUTestApp->freezePoints[ *((int*)clientData) ].z=1;
		curGPUTestApp->freezePoints[ *((int*)clientData) ].w=1;
	}
	
protected:
	virtual void keyboard(unsigned char key);
	virtual void keyboardSpecial(int key);
	virtual void mouse(int button, int state, int x, int y);
	virtual void mouseMove(int x, int y);
	virtual void mousePassiveMove(int x, int y);

	virtual void reshape(int w, int h);

	virtual void idle() ;
	virtual void display() ;
	virtual void init();

public:

	GLAppGPUTest(const char *title, int w, int h, int flags = 0);
	GLAppGPUTest(int *argcp, char **argv, const char *title, int w, int h, int flags = 0);

	void setPerspectiveProj( GLdouble fovY, GLdouble aspect, GLdouble zNear, GLdouble zFar );

};

#endif
