///////////////////////////////////////////////////////////////////////////////
/*! \file Lighting.h
 *  \brief Framework Nemo
 *  \author C. Crassin
 *
 * $Id: ShaderProgramGLSL.h,v 1.2 2007/09/14 11:54:56 gascuel Exp $
 */
///////////////////////////////////////////////////////////////////////////////
// History:
//
// 2005-01-01 : Creation.

#ifndef ShaderProgramGLSL_h
#define ShaderProgramGLSL_h

#include "Vector.h"
#include "Matrix.h"
#include "ShaderProgram.h"

#include <string>
#include <iostream>
#include <fstream>
#include <map>
#include <list>

namespace NemoGraphics {

#define NG_MAX_ERROR_BUFF_SIZE 4096

class ShaderGLSL : public Shader {	
	friend class ShaderProgramGLSL;

	GLuint shaderID;

	char errorBuff[NG_MAX_ERROR_BUFF_SIZE];

	void checkShaderErrors();


	static std::string manageIncludes(std::string src, std::string sourceFileName);

	///////////////////////////////////////////////////
	
	struct MacroStruct{std::string name; std::string value;};
	std::map<unsigned long, MacroStruct> macrosList;
public:

	ShaderGLSL(const char *src, ShaderType st);
	virtual ~ShaderGLSL(){}

	virtual bool compile();

	virtual void defineMacro(const char *macro, const char *value);
	virtual void defineMacro(const char *macro, int val);
	virtual void defineMacro(const char *macro, float val);

	virtual void reload(){
		this->shaderSource=this->loadSource(this->sourceFileName.c_str());

		for(std::map<unsigned long, MacroStruct>::iterator it=this->macrosList.begin(); it!=this->macrosList.end(); it++){
			this->defineMacro(it->second.name.c_str(), it->second.value.c_str());
		}
		sourceChanged=true;
	}

	bool hasMain();
};

class ShaderProgramGLSL : public ShaderProgram {
	GLuint programID;
	bool myProgID;

	//struct SamplerStruct{std::string name; GLint location; GLuint type; GLuint texID; GLuint texUnit;};
	struct SamplerStruct{std::string name; GLint location; Texture *tex; GLuint texUnit;};

	std::map<GLint, SamplerStruct> samplersInfo;

	bool isCompiled;

	char errorBuff[NG_MAX_ERROR_BUFF_SIZE];
	void checkProgramErrors(GLuint stat);
	

	bool binded;

	std::list<Shader *> shadersList;

	int texUnitsLowerLimit;


	////////////////////////////
	GLuint globalShaderID[3];
	std::string globalShaderSource[3];
	std::list<Shader *> globalShadersList[3];
	bool globalShaderAttached[3];


	///////////////////////////

	std::map<GLint, GLint> vertexAttribList;
public:
	ShaderProgramGLSL(GLuint progid=0);
	~ShaderProgramGLSL();

	void compile();

	void bind(bool render=true);
	void unbind(bool render=true);

	void addShader(Shader *shader);

	int getProgramID(){
		return this->programID;
	}

	void setTexUnitsLowerLimit(int l){
		this->texUnitsLowerLimit=l;
	}

	//Parameters setting//
	int getParamLocation(const char *pname);

	void setParam(const char *pname, float val);

	void setParam(const char *pname, const Vector2f &val);
	void setParam(const char *pname, const Vector3f &val);
	void setParam(const char *pname, const Vector4f &val);

	void setParam(const char *pname, int v);
	void setParam(const char *pname, const Vector3i &val);

	void setParam(const char *pname, const Mat4f &val);

	//void setSamplerParam(const char *pname, GLuint texid, GLuint stype);
	void setSamplerParam(const char *pname, Texture *tex);

	void setFragDataLocation(const char *pname, GLuint colnum);

	void setProgramParam(GLuint pname, GLint val);

	//////////////////
	void setAttrib(const char *aname, Vector3f *data);
	void setAttrib(const char *aname, Vector4f *data);
};

};
#endif
