///////////////////////////////////////////////////////////////////////////////
/*! 
 *  \file GPUTest_FS.glsl
 *  \brief FragSniffer Fragment Shader.
 *  
 *  The Froggy FragSniffer.
 *	This application has been made for testing GPU fragment scheduling among 
 *  threads processors.
 *  \author C. Crassin, cyril.crassin@inrialpes.fr
 *	\date 05/03/2008
 *  \version 0.2
 */
///////////////////////////////////////////////////////////////////////////////

#version 120
#extension GL_EXT_gpu_shader4 : enable

#pragma optionNV(unroll all) 
//#pragma optionNV(ifcvt none)

//#pragma optionNV(fastmath on)
//#pragma optionNV(fastprecision on)
//#pragma optionNV(inline all)
//#pragma optionNV(strict on)

#define ENABLE_BLOCKING 1

#define USE_BLOCKING_SQUARE 0
#define USE_BLOCKING_QUADPOINTS 0
#define USE_BLOCKING_TWOPOINTS 1

#define USE_BAD_LOOP 0

#define NUM_FREEZE_POINTS 4
#define ENABLE_FREEZE_POINT_0 0
#define ENABLE_FREEZE_POINT_1 0
#define ENABLE_FREEZE_POINT_2 0
#define ENABLE_FREEZE_POINT_3 0

#define NUM_PIPEVARS 40

#define SHOW_GRID 1

uniform vec4 userParam;
uniform vec4 userPos;
uniform vec4 freezePoints[NUM_FREEZE_POINTS];
uniform vec4 userColor;


//Stress loop, slow down fragments rendering
void stressLoop(inout vec4 color, in vec3 dcol, in float numloop){

//Bad loop tend to use many registers to prevent threads scheduling on Multi Processor (as threads are shared inside blocks).
#if USE_BAD_LOOP
		vec4 cols[NUM_PIPEVARS];
		for(float i=0.0; i<numloop; i+=1.0){

			cols[0]=cols[0]+userParam.w+cols[NUM_PIPEVARS-1];

			for(int pv=1; pv<NUM_PIPEVARS-1; pv++)
				cols[pv]=cols[pv]+userParam.w*cols[(pv+10)%NUM_PIPEVARS];

			cols[NUM_PIPEVARS-1]=cols[NUM_PIPEVARS-1]+userParam.w+cols[0];
		}

		float alpv=0;
		for(int pv=0; pv<NUM_PIPEVARS-1; pv++){
			alpv+=cols[pv].r;
			alpv+=cols[pv].g;
			alpv+=cols[pv].b;
			alpv+=cols[pv].a;
		}
		color=vec4(dcol, alpv);

#else
		float col=0.0;
		for(float i=0.0; i<numloop; i+=1.0){
			col=col+userParam.w;
		}

		color=vec4(dcol, col);
#endif
}


void main(void)
{
	
	float col=0.0;
	vec4 color=vec4(1.0,0.0,0.0,1.0);

	gl_FragColor=vec4(1.0,0.0,0.0,1.0);

#if ENABLE_BLOCKING

#if ENABLE_FREEZE_POINT_0
	if( gl_FragCoord.x>=freezePoints[0].x && gl_FragCoord.x<freezePoints[0].x+freezePoints[0].z && 
		gl_FragCoord.y>=freezePoints[0].y && gl_FragCoord.y<freezePoints[0].y+freezePoints[0].w ){

		stressLoop(color, vec3(1.0,0.0,0.0), userParam.x);
	}else 
#endif
#if ENABLE_FREEZE_POINT_1
	if( gl_FragCoord.x>=freezePoints[1].x && gl_FragCoord.x<freezePoints[1].x+freezePoints[1].z && 
		gl_FragCoord.y>=freezePoints[1].y && gl_FragCoord.y<freezePoints[1].y+freezePoints[1].w ){

		stressLoop(color, vec3(0.9,0.0,0.0), userParam.x);
	}else 
#endif
#if ENABLE_FREEZE_POINT_2
	if( gl_FragCoord.x>=freezePoints[2].x && gl_FragCoord.x<freezePoints[2].x+freezePoints[2].z && 
		gl_FragCoord.y>=freezePoints[2].y && gl_FragCoord.y<freezePoints[2].y+freezePoints[2].w ){

		stressLoop(color, vec3(0.8,0.0,0.0), userParam.x);
	}else 
#endif
#if ENABLE_FREEZE_POINT_3
	if( gl_FragCoord.x>=freezePoints[3].x && gl_FragCoord.x<freezePoints[3].x+freezePoints[3].z && 
		gl_FragCoord.y>=freezePoints[3].y && gl_FragCoord.y<freezePoints[3].y+freezePoints[3].w ){

		stressLoop(color, vec3(0.7,0.0,0.0), userParam.x);
	}else 
#endif

#endif ///ENABLE_BLOCKING

	////Unfreezed fragments////
	{
		stressLoop(color, gl_Color.xyz, userParam.y);


#if SHOW_GRID //Grid
		 if( ((int(gl_FragCoord.x)%8)*(int(gl_FragCoord.y)%4))==0 )
			color = vec4(vec3(0.8), 1.0);
		else{
			if( ( ((int(gl_FragCoord.x)/16)+(int(gl_FragCoord.y)/16)) ) % 2 != 0)
				color =  vec4(vec3(1.0, 1.0, 1.7)-color.rgb, color.w);

			if( ( ( /*(int(gl_FragCoord.x)/32)+*/(int(gl_FragCoord.y)/32)) ) % 2 != 0)
				color =  vec4(color.rgb*vec3(0.9), color.w);
		}
#endif

	}


	gl_FragColor=color;
}